if(not(SB_Actions)) then
  SB_Actions = {};
end

if(not(SB_Bindings)) then
  SB_Bindings = {}; --table that holds all the bindings
end

if(type(SB_Bindings['version']) == 'number') then
  SB_Bindings['version'] = nil;
end

local function SB_Args_to_Table(...) --this takes the arguments from GetBinding and turns them into a table
   local table, var;
   table = {};
   for i=1, select("#",...) do
      var = select(i,...);
      table[i] = var;
   end
   return table;
end

local function SB_GetBinding(index) --this returns a table containing the key binding, with the command first, followed by each keybind
  --[[ local binding;
   binding = {}; 
   binding = SB_Args_to_Table(GetBinding(index));
   return binding;]]
   return SB_Args_to_Table(GetBinding(index));
end

local function SB_SetBinding(binding)
   if(#binding > 1) then
      local command = binding[1];
      --print("Setting keybinds for "..command..":");
      for i=2,#binding do
         SetBinding(binding[i],command);
      end
   end
end

local function SB_UnsetBinding(binding)
   if(#binding > 1) then
      for i=2, #binding do
         SetBinding(binding[i]);
      end
   end
end

local function SB_RemoveBindings()
   local bNum;
   bNum = GetNumBindings();
   for i=1, bNum do
      SB_UnsetBinding(SB_GetBinding(i));
   end
end

local function SB_StoreBindings(table) --this stores the current bindings into a table
   local bNum; --number of bindings
   bNum = GetNumBindings();
   for i=1, bNum do
      table[i] = SB_GetBinding(i);
   end
   return true;
end

local function SB_RestoreBindings(bindings) --this sets the current keybindings to the ones listed in the table
   SB_RemoveBindings();
   for i=1, #bindings do
      SB_SetBinding(bindings[i])
   end
   SaveBindings(GetCurrentBindingSet());
   return true;
end

local function SB_GetAction(index)
   local action;
   action = {};
   action = SB_Args_to_Table(GetActionInfo(index));
   if(#action < 1) then
      action = nil;
   end
   return action;
end

local function SB_UnsetAction(index)
   PickupAction(index);
   ClearCursor();
end

local function SB_SetCompanion(index,action)
   PickupCompanion(action[3],action[2]);
end

local function SB_SetEquipmentSet(index,action)
   PickupEquipmentSetByName(action[2]);
end

local function SB_SetItem(index,action)
   PickupItem(action[2]);
end

local function SB_SetMacro(index,action)
   PickupMacro(action[2]);
end

local function SB_SetSpell(index,action)
   local name = GetSpellName(action[2],action[4]);
   PickupSpell(name);
end

local function SB_SetAction(index,action)
   if(HasAction(index)) then
      SB_UnsetAction(index);
   end
   if(action[1] == 'spell') then
      SB_SetSpell(index,action);
   elseif(action[1] == 'equipmentset') then
      SB_SetEquipmentSet(index,action);
   elseif(action[1] == 'item') then
      SB_SetItem(index,action);
   elseif(action[1] == 'macro') then
      SB_SetMacro(index,action);
   elseif(action[1] == 'companion') then
      SB_SetCompanion(index,action);
   end
   PlaceAction(index);
end

local function SB_StoreActions(table)
   for i=1,120 do
      table[i] = SB_GetAction(i);
   end
   return true;
end


local function SB_RestoreActions(table)
   for i=1,120 do
      ClearCursor();
      if(table[i]) then
         SB_SetAction(i,table[i]);
      elseif(HasAction(i)) then
         SB_UnsetAction(i);
      end
   end
   return true;
end

local function printtable(table)
   if(type(table) == 'table') then
      for i=1,#table do
         print(table[i]);
      end
   else
      print(table);
   end
end

local function SB_Save_Handler(msg)
   msg = msg:trim();
   if((msg ~= '') and (msg ~= 'version')) then
      msg = strlower(msg);
      SB_Bindings[msg] = {};
      SB_Actions[msg] = {};
      if(SB_StoreBindings(SB_Bindings[msg]) and SB_StoreActions(SB_Actions[msg])) then
         print("|cFF00FF00[Swapbar]|cFFFFFFFF  Profile "..msg.." saved.");
      end
   else
      SB_PrintHelp();
   end
end

local function SB_Load_Handler(msg)
   msg = msg:trim();
   if(InCombatLockdown()) then
      print("|cFF00FF00[Swapbar]|cFFFFFFFF Cannot load profiles in combat.");
   else
      if((msg ~= '') and (msg ~= 'version')) then
         msg = strlower(msg);
         if(SB_Bindings[msg] and SB_Actions[msg]) then
            local soundenabled;
            soundenabled = GetCVar("Sound_EnableAllSound");
            SetCVar("Sound_EnableAllSound",0);
            if(SB_RestoreBindings(SB_Bindings[msg]) and SB_RestoreActions(SB_Actions[msg])) then
               print("|cFF00FF00[Swapbar]|cFFFFFFFF Profile "..msg.." loaded.");
            end
            SetCVar("Sound_EnableAllSound",soundenabled);
         else
            print("|cFF00FF00[Swapbar]|cFFFFFFFF No such profile exists.("..msg..")");
         end
      else
         SB_PrintHelp();
      end
   end
end

local function SB_PrintHelp()
   print("|cFF00FF00SwapBar commands usage:");
   print("- To save a profile: /sbsave <name>");
   print("- To load a profile: /sbload <profile");
end

local function SB_PrintProfiles()
  print("|cFF00FF00SwapBar Profiles:");
  for key, value in next, SB_Actions do
    print(key);
  end
end

local function SB_RemoveProfile(msg)
  if(SB_Actions[msg] and SB_Bindings[msg]) then
    wipe(SB_Actions[msg]);
    wipe(SB_Bindings[msg]);
    SB_Actions[msg] = nil;
    SB_Bindings[msg] = nil;
    print("|cFF00FF00[SwapBar]|cFFFFFFFF Profile "..msg.." removed.");
  else
    print("|cFF00FF00[SwapBar]|cFFFFFFFF No such profile "..msg);
  end
end

SLASH_SB_SAVE1 = "/sbsave";
SLASH_SB_LOAD1 = "/sbload";
SLASH_SB_HELP1 = "/sbhelp";
SLASH_SB_LIST1 = "/sblist";
SLASH_SB_REMOVE1 = "/sbremove";
SlashCmdList["SB_SAVE"] = SB_Save_Handler;
SlashCmdList["SB_LOAD"] = SB_Load_Handler;
SlashCmdList["SB_HELP"] = SB_PrintHelp;
SlashCmdList["SB_LIST"] = SB_PrintProfiles;
SlashCmdList["SB_REMOVE"] = SB_RemoveProfile;
